import java.util.Random;
import java.lang.ref.SoftReference;
import java.lang.ref.ReferenceQueue;

class TextPage
{
    char[] symbols = new char[300];
}

class Book
{
    String      name;
    TextPage    pages[] = new TextPage[100];

    public Book ()
    {
        for (int i = 0; i < pages.length; i++)
            pages[i] = new TextPage();
    }
}

class BookShelf
{
    String subject;
    Book[] books = new Book[100];

    public BookShelf (String subject)
    {
        this.subject = subject;
        for (int i = 0; i < books.length; i++)
            books[i] = new Book();

        if (subject.equals("Sports"))
            books[0].name = "History of Ping Pong";
    }
}

class BookShelfReference extends SoftReference
{
    String subject;
    static ReferenceQueue collectedQueue = new ReferenceQueue();

    public BookShelfReference (BookShelf shelf)
    {
        super(shelf, collectedQueue);
        this.subject = shelf.subject;
    }
}

class GarbageMonitor extends Thread
{
    ReferenceQueue queue;

    public GarbageMonitor (ReferenceQueue queue)
    {
        this.queue = queue;
    }

    public void run ()
    {
        while (true)
        {
            try
            {
                BookShelfReference shelfRef =
                    (BookShelfReference) queue.remove(5000);
                System.out.println("Monitor: Shelf subject '" +
                     shelfRef.subject + "' was collected.");
            }
            catch (Exception e)
            {   break;  }
        }
    }
}

public class Library
{
    public static void main (String args[])
    {
        String subjects[] = { "Sports", "New Age", "Religion",
                            "Sci-Fi", "Romance", "Do-it-yourself",
                            "Cooking", "Gardening", "Travel",
                            "Mystery", "Fantasy", "Computers",
                            "Business", "Young readers", "JW Books" };

GarbageMonitor monitor =
    new GarbageMonitor(BookShelfReference.collectedQueue);
monitor.start();

        BookShelfReference[] shelves = new BookShelfReference[25];
        int checkIndex = 10 + getRandomIndex(15);

        for (int i = 0; i < shelves.length; i++)
        {
            String subject = subjects[getRandomIndex(subjects.length)];
            System.out.println("Creating bookshelf: " + (i + 1) +
                               ", subject: " + subject);
            // Keep track of shelves as SoftReference objects
            shelves[i] = new BookShelfReference(new BookShelf(subject));

            // at a random interval check to see where the last
            //  "Sports" shelf was
            if (i == checkIndex)
            {
                System.out.println("Checking for Sports...");
                for (int j = i; j >= 0; j--)
                {
                    // get BookShelf reference from SoftReference
                    BookShelf shelf = (BookShelf)shelves[j].get();
                    if (shelf != null)
                    {
                        if (shelf.subject.equals("Sports"))
                        {
                            System.out.println();
                            System.out.println("Shelf " + (j + 1) +
                                               " is Sports");
                            System.out.println("First book is: " +
                                               shelf.books[0].name);
                            System.out.println();
                            break;
                        }
                        else
                            System.out.println("No match. " +
                                "Subject was: " + shelf.subject);
                    }
                }
            }
        }
    }

    private static Random rnd = new Random(System.currentTimeMillis());
    private static int getRandomIndex (int range)
    {
        return ((Math.abs(rnd.nextInt())) % range);
    }
}
