import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.awt.image.*;
import java.io.*;

public class ImageCanvas extends Canvas
                         implements ClipboardOwner, KeyListener
{
    Clipboard       clipboard;
    Image           img;
    MediaTracker    tracker = new MediaTracker(this);

    public ImageCanvas ()
    {
        addKeyListener(this);
        clipboard = getToolkit().getSystemClipboard();
    }

    public void setImage (Image img)
    {
        tracker.addImage(img, 0);
        try
        {
            tracker.waitForAll();
        }
        catch (Exception e)
        {
            System.out.println("Error loading image: " + e.getMessage());
            return;
        }

        if (tracker.isErrorAny())
        {
            System.out.println("Error loading image.");
            return;
        }

        this.img = img;
        repaint();
    }

    public void paint (Graphics g)
    {
        if (img != null)
        {
            Dimension d = this.getSize();
            g.setColor(Color.white);
            g.fillRect(0,0, d.width, d.height);     // blank background
            g.drawImage(img, 0, 0, img.getWidth(this), img.getHeight(this), this);
        }
    }

    public void keyPressed (KeyEvent evt)
    {
        if (evt.isControlDown())
        {
            if (evt.getKeyCode() == KeyEvent.VK_A)  // copy command
            {
                try
                {
                    //clipboard.setContents(new TransferableImage(img, this), this);
                    int width = img.getWidth(this);
                    int height = img.getHeight(this);
                    int[] pixels = new int[width * height];
                    PixelGrabber pg = new PixelGrabber(img, 0, 0, width, height, pixels, 0, width);
                    pg.grabPixels(0);
                    byte[] imageData = new byte[(pixels.length + 2) * 6];

                    imageData[0] = (byte) ((width >> 30) + 32);
                    imageData[1] = (byte) ((width >> 24) + 32);
                    imageData[2] = (byte) ((width >> 18) + 32);
                    imageData[3] = (byte) ((width >> 12) + 32);
                    imageData[4] = (byte) ((width >> 6) + 32);
                    imageData[5] = (byte) (width + 32);

                    imageData[6] = (byte) ((height >> 30) + 32);
                    imageData[7] = (byte) ((height >> 24) + 32);
                    imageData[8] = (byte) ((height >> 18) + 32);
                    imageData[9] = (byte) ((height >> 12) + 32);
                    imageData[10] = (byte) ((height >> 6) + 32);
                    imageData[11] = (byte) (height + 32);

                    int offset = 2;
                    for (int i = offset; i < (pixels.length + 2); i++)
                    {
                        imageData[i * 6] = (byte) ((pixels[i - offset] >> 30) + 32);
                        imageData[i * 6 + 1] = (byte) ((pixels[i - offset] >> 24) + 32);
                        imageData[i * 6 + 2] = (byte) ((pixels[i - offset] >> 18) + 32);
                        imageData[i * 6 + 3] = (byte) ((pixels[i - offset] >> 12) + 32);
                        imageData[i * 6 + 4] = (byte) ((pixels[i - offset] >> 6) + 32);
                        imageData[i * 6 + 5] = (byte) (pixels[i - offset] + 32);
                    }
                    StringSelection sClipboardText =
                                            new StringSelection(new String(imageData));
                    clipboard.setContents(sClipboardText, this);
                }
                catch (Exception e)
                {
                    System.out.println("Error copying data. Error: " + e.getMessage());
                }
                evt.consume();
            }
            else if (evt.getKeyCode() == KeyEvent.VK_B) // paste command
            {
                Transferable clipboardData = clipboard.getContents(this);
                if (clipboardData != null)
                {
                    try
                    {
                        DataFlavor[] flavors = clipboardData.getTransferDataFlavors();
                        for (int j = 0; j < flavors.length; j++)
                        {
                            System.out.println(flavors[j].getMimeType());
                            System.out.println(flavors[j].getHumanPresentableName());
                        }
                        String serializedData =
                                    (String)clipboardData.getTransferData(DataFlavor.stringFlavor);
                        byte[] imageData = serializedData.getBytes();
                        int pixels[] = new int[(imageData.length / 6) - 2];
                        int width = ((imageData[0] - 32) << 30) | ((imageData[1] - 32) << 24) | ((imageData[2] - 32) << 18) | ((imageData[3] - 32) << 12)  | ((imageData[4] - 32) << 6)  | (imageData[5] - 32);
                        int height = ((imageData[6] - 32) << 30) | ((imageData[7] - 32) << 24) | ((imageData[8] - 32) << 18) | ((imageData[9] - 32) << 12)  | ((imageData[10] - 32) << 6)  | (imageData[11] - 32);

                        int offset = 12;
                        for (int i = 0; i < pixels.length; i++)
                        {
                            pixels[i] = ((imageData[i * 6 + offset] - 32) << 30) | (imageData[i * 6 + offset + 1] - 32) << 24 |
                                        ((imageData[i * 6 + offset + 2] - 32) << 18) | (imageData[i * 6 + offset + 3] - 32) << 12 |
                                        ((imageData[i * 6 + offset + 4] - 32) << 6) | imageData[i * 6 + offset + 5] - 32;
                        }
                        MemoryImageSource mis = new MemoryImageSource(width, height, pixels, 0, width);
                        this.setImage(Toolkit.getDefaultToolkit().createImage(mis));
                    }
                    catch (Exception e)
                    {
                        System.out.println("Error pasting data. Error: " + e.getMessage());
                    }
                }
                evt.consume();
            }
        }
    }

    public void keyReleased (KeyEvent evt)  {   }
    public void keyTyped (KeyEvent evt)     {   }

    public void lostOwnership(Clipboard clipboard, Transferable contents)
    {   }

    public static void main (String args[])
    {
        Frame f = new Frame("Image Copy");
        f.setLayout(new GridLayout(0,1));

        ImageCanvas canvas1 = new ImageCanvas();
        ImageCanvas canvas2 = new ImageCanvas();

        canvas1.setImage(Toolkit.getDefaultToolkit().getImage("images/cloud.gif"));
        canvas2.setImage(Toolkit.getDefaultToolkit().getImage("images/sun.gif"));

        f.add(canvas1);
        f.add(canvas2);

        f.setSize(300,300);
        f.show();
    }

}

class TransferableImage implements Transferable
{
    DataFlavor  imageFlavor;
    Image       img;
    ImageData   imageData = new ImageData();

    public TransferableImage (Image img, Component c) throws Exception
    {
        this.img = img;
        /*
        int[] pixels = new int[img.getWidth(c) * img.getHeight(c)];
        PixelGrabber pg = new PixelGrabber(img, 0, 0, img.getWidth(c), img.getHeight(c), pixels, 0, img.getWidth(c));
        pg.grabPixels(0);
        imageData.pixels = new char[pixels.length * 2 + 4];
        for (int i = 0; i < pixels.length; i++)
        {
            imageData.pixels[i * 2] = (pixels[i] >> 16) & 0xffff
            imageData.pixels[i * 2 + 1] = pixels[i] & 0xffff;
        }
        imageData.width = img.getWidth(c);
        imageData.height = img.getHeight(c);

        StringSelection sClipboardText =
                                new StringSelection(sCopyText);
        imageFlavor = new DataFlavor(imageData.getClass(), "Java Image");
        */
    }

    public Object getTransferData(DataFlavor flavor)
    {
        if (imageFlavor.equals(flavor))
            return (imageData);
        else
            return (null);
    }

    public DataFlavor[] getTransferDataFlavors()
    {
        DataFlavor[] flavors = new DataFlavor[1];
        flavors[0] = imageFlavor;
        return (flavors);
    }

    public boolean isDataFlavorSupported(DataFlavor flavor)
    {
        return (imageFlavor.equals(flavor));
    }
}

class ImageData implements Serializable
{
    //public int[]        pixels;
    public char[]       pixels;
    public int          width;
    public int          height;
}
