import java.awt.datatransfer.*;
import java.awt.Toolkit;
import java.io.IOException;
import java.util.*;

public class NativeExtendedClipboard implements ExtendedClipboard
{
    static
    {
        System.loadLibrary("NativeExtendedClipboard");
    }

    private Clipboard javaSystemClipboard;
    private Map       systemFlavorMap;
    //private Vector    mappedMimeTypes = new Vector();

    public NativeExtendedClipboard ()
    {
        systemFlavorMap = SystemFlavorMap.getDefaultFlavorMap().getNativesForFlavors(null);
        /*
        Map systemFlavorMap = SystemFlavorMap.getDefaultFlavorMap().getNativesForFlavors(null);
        Iterator it = systemFlavorMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry) it.next();
            String key = (String) entry.getKey();
            System.out.println("key: " + key);
            System.out.println("mimeType: " + key.substring(0, key.indexOf(';')));
            mappedMimeTypes.add(key.substring(0, key.indexOf(';')));
        }*/
    }

    public ExtendedTransferable getContents (Object requestor)
    {
        Clipboard javaClipboard = getJavaSystemClipboard();
        return (new NativeExtendedTransferable(javaClipboard.getContents(requestor)));
    }

    public void setContents (ExtendedTransferable contents, ClipboardOwner owner)
    {
        if (hasNativeMap(contents))
        {
            System.out.println("1");
            getJavaSystemClipboard().setContents(contents, owner);
            System.out.println("2");
        }
        else
        {
            System.out.println("3");
            setNativeTransferable(contents);
            System.out.println("4");
        }
    }

    private Clipboard getJavaSystemClipboard ()
    {
        if (javaSystemClipboard == null)
            this.javaSystemClipboard = Toolkit.getDefaultToolkit().getSystemClipboard();

        return (this.javaSystemClipboard);
    }

    private boolean hasNativeMap (ExtendedTransferable contents)
    {
        DataFlavor[] flavors = contents.getTransferDataFlavors();
        for (int i = 0; i < flavors.length; i++)
        {
            if (systemFlavorMap.containsKey(flavors[i].getMimeType()))
                return (true);
            /*
            String mimeType = flavors[i].getMimeType();
            System.out.println("mimeType: " + mimeType);
            for (int j = 0; j < mappedMimeTypes.size(); j++)
            {
                System.out.println("check against: " + mappedMimeTypes.get(j));
                if (mappedMimeTypes.get(j).equals(mimeType))
                    return (true);
            }*/
        }

        return (false);
    }

    private void setNativeTransferable (ExtendedTransferable contents)
    {
        int[] formats = contents.getFormats();
        for (int i = 0; i < formats.length; i++)
            setNativeClipboardData(formats[i], contents.getBytes(formats[i]));
    }

    public int registerNewFlavor (DataFlavor flavor)
    {
        return (registerNativeFormat(flavor.getMimeType()));
    }

    private native int registerNativeFormat (String formatDescription);

    private native void setNativeClipboardData (int format, byte[] bytes);
}

class NativeExtendedTransferable implements ExtendedTransferable
{
    Transferable t;

    public NativeExtendedTransferable (Transferable t)
    {
        this.t = t;
    }

    public native int[] getFormats ();

    public native byte[] getBytes (int format);

    public Object getTransferData(DataFlavor flavor) throws UnsupportedFlavorException, IOException
    {
        if (t != null)
            return (t.getTransferData(flavor));
        else
            return (null);
    }

    public DataFlavor[] getTransferDataFlavors()
    {
        if (t != null)
            return (t.getTransferDataFlavors());
        else
            return (null);
    }

    public boolean isDataFlavorSupported(DataFlavor flavor)
    {
        if (t != null)
            return (t.isDataFlavorSupported(flavor));
        else if (flavor.equals(ExtendedClipboard.extendedFlavor))
            return (true);
        else
            return (false);
    }
}
