import java.awt.datatransfer.*;
import java.awt.Toolkit;
import java.io.IOException;
import java.util.*;

public class NativeExtendedClipboard extends Clipboard
                                    implements Transferable
{
    static
    {
        System.loadLibrary("NativeExtendedClipboard");
    }

    private Map          nativeFlavorMap;
    private Vector       nativeFlavors = new Vector();

    public NativeExtendedClipboard ()
    {
        super("NativeClipboard");
        createNativeFlavorMap();
    }

    private void createNativeFlavorMap ()
    {
        nativeFlavorMap = new HashMap();
        DataFlavor flavorImageRaw = null;

        try
        {
            flavorImageRaw = new DataFlavor("image/raw");
            nativeFlavors.add(flavorImageRaw);

            // create map of mimeType/adaptor objects
            nativeFlavorMap.put(flavorImageRaw.getMimeType(),
             Class.forName("NativeRawImageAdaptor").newInstance());
        }
        catch (Exception cnfe)
        {
            System.out.println("Errors during createNativeFlavorMap(): " + cnfe.getMessage());
        }
    }

    public Object getTransferData (DataFlavor flavor)
                             throws UnsupportedFlavorException
    {
        Iterator it = nativeFlavorMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry) it.next();
            String key = (String) entry.getKey();

            if (flavor.getMimeType().equals(key))
            {
                NativeClipboardAdaptor nca =
                    (NativeClipboardAdaptor) entry.getValue();
                return (nca.getClipboardData());
            }
        }

        throw new UnsupportedFlavorException (flavor);
    }

    public boolean isDataFlavorSupported (DataFlavor flavor)
    {
        int numNativeFlavors = nativeFlavors.size();
        for (int i = 0; i < numNativeFlavors; i++)
        {
            DataFlavor flavorToCheck =
                (DataFlavor) nativeFlavors.elementAt(i);
            if (flavorToCheck.equals(flavor))
                return (true);
        }

        return (false);
    }

    public DataFlavor[] getTransferDataFlavors()
    {
        return ((DataFlavor[]) nativeFlavors.toArray());
    }

    public Transferable getContents (Object requestor)
    {
        int[] formats = getFormats();

        Iterator it = nativeFlavorMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry) it.next();
            NativeClipboardAdaptor nca =
                  (NativeClipboardAdaptor) entry.getValue();

            for (int i = 0; i < formats.length; i++)
            {
                if (nca.getFormat() == formats[i])
                {
                    nca.setBytes(getBytes(formats[i]));
                    break;
                }
            }
        }

        return (this);
    }

    public void setContents (Transferable contents, ClipboardOwner owner)
    {
        setNativeTransferable(contents);
    }

    private void setNativeTransferable (Transferable contents)
    {
        DataFlavor[] flavors = contents.getTransferDataFlavors();
        String       className = "";
        boolean      foundIt = false;

        Iterator it = nativeFlavorMap.entrySet().iterator();
        while (it.hasNext())
        {
            Map.Entry entry = (Map.Entry) it.next();
            String key = (String) entry.getKey();

            // find first match and use it
            for (int i = 0; i < flavors.length; i++)
            {
                if (flavors[i].getMimeType().equals(key))
                {
                    NativeClipboardAdaptor nca =
                        (NativeClipboardAdaptor) entry.getValue();
                    nca.setTransferable(contents);
                    setNativeClipboardData(
                                nca.getFormat(), nca.getBytes());
                    foundIt = true;
                    break;
                }
            }

            if (foundIt)
                break;
        }
    }

    private native int[] getFormats ();
    private native byte[] getBytes (int format);
    private native void setNativeClipboardData (int format, byte[] bytes);
}
