import javax.swing.*;
import java.net.*;
import java.awt.event.*;
import java.awt.print.*;
import java.awt.*;
import java.io.*;

public class JEditorPaneFrame extends JFrame
                            implements ActionListener
{
    JMenuItem print = new JMenuItem("Print");
    JMenuItem exit = new JMenuItem("Exit");
    MyEditor editor;

    public JEditorPaneFrame (String title)
    {
        super(title);

        try
        {
            print.addActionListener(this);
            exit.addActionListener(this);
            JMenu file = new JMenu("File");
            file.add(print);
            file.add(exit);
            JMenuBar menuBar = new JMenuBar();
            menuBar.add(file);
            setJMenuBar(menuBar);

            URL url =
                new URL("file:/d:/jdk1.3/docs/guide/2d/index.html");
            editor = new MyEditor(url);

            JScrollPane sp = new JScrollPane(editor);

            editor.setEditable(false);
            getContentPane().add("Center", sp);
        }
        catch (Exception e)
        {
            System.out.println("Error: " + e.getMessage());
            System.exit(1);
        }

        setSize(400,400);
        setVisible(true);
    }

    public void actionPerformed(ActionEvent evt)
    {
        if (evt.getSource() == print)
        {
            PrinterJob job = PrinterJob.getPrinterJob();
            PageFormat pf = job.defaultPage();

            job.setPrintable(editor);
            try
            {
                job.print();
            }
            catch (PrinterException pe)
            {
                System.out.println("Printer Error: " +
                                            pe.getMessage());
            }
        }
        else if (evt.getSource() == exit)
        {
            this.dispose();
            System.exit(1);
        }
    }

    public static void main (String args[])
    {
        new JEditorPaneFrame("Test JEditor Pane Print");
    }
}

class MyEditor extends JEditorPane implements Printable
{
    int numPages;
    int pageWidth;
    int pageHeight;

    public MyEditor (URL url) throws IOException
    {
        super (url);
    }

    public int print (Graphics graphics, PageFormat pageFormat,
                                                       int pageIndex)
    {
        int screenResolution = getToolkit().getScreenResolution();
        double pixelsPerPoint = (double) screenResolution / 72d;

        int keepWidth = getSize().width;
        int keepHeight = getSize().height;

        if (pageIndex == 0)
        {
            // first time through - lets calculate num pages
            pageWidth = (int) (pageFormat.getImageableWidth() *
                                                    pixelsPerPoint);
            pageHeight = (int) (pageFormat.getImageableHeight() *
                                                    pixelsPerPoint);

            setSize(pageWidth, pageHeight);
            Graphics temp = graphics.create();

            printAll(temp); // recalculate our preferred height

            temp = null;

            int newHeight = getPreferredSize().height;
            if (newHeight % pageHeight == 0)
                numPages = newHeight / pageHeight;
            else
                numPages = newHeight / pageHeight + 1;
        }
        else if (pageIndex >= numPages)
        {
            return (Printable.NO_SUCH_PAGE);
        }

        // make translation between pixels and points
        int newXOrigin = (int) (pageFormat.getImageableX() *
                                                    pixelsPerPoint);
        int newYOrigin = (int) (pageFormat.getImageableY() *
                                                    pixelsPerPoint);

        setSize(pageWidth, pageHeight);

        if (graphics instanceof Graphics2D)
        {
            Graphics2D graphics2D = (Graphics2D) graphics;
            graphics2D.scale(1 / pixelsPerPoint, 1 / pixelsPerPoint);
        }

        graphics.translate(newXOrigin,
                        newYOrigin - (pageIndex * pageHeight));
        graphics.setClip(0,(pageIndex * pageHeight),
                                        pageWidth, pageHeight);

        printAll(graphics);

        setSize(keepWidth, keepHeight);
        return (Printable.PAGE_EXISTS);
    }
}